@@ -1,32 +1,28 @@
-# Copyright (c) 2006, 2008-2011, 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2012-2014 Google, Inc.
-# Copyright (c) 2013-2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2014 Michal Nowikowski <godfryd@gmail.com>
-# Copyright (c) 2014 Brett Cannon <brett@python.org>
-# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
-# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
-# Copyright (c) 2016 Alexander Todorov <atodorov@otb.bg>
-# Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
-# Copyright (c) 2018 Lucas Cimon <lucas.cimon@gmail.com>
-# Copyright (c) 2018 Natalie Serebryakova <natalie.serebryakova@Natalies-MacBook-Pro.local>
-# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
-# Copyright (c) 2019, 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
-# Copyright (c) 2019 Robert Schweizer <robert_schweizer@gmx.de>
-# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
-
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
+
+"""Check for new / old style related problems."""
+
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
 
-"""check for new / old style related problems
-"""
 import astroid
+from astroid import nodes
 
 from pylint.checkers import BaseChecker
-from pylint.checkers.utils import check_messages, has_known_bases, node_frame_class
-from pylint.interfaces import IAstroidChecker
+from pylint.checkers.utils import (
+    has_known_bases,
+    node_frame_class,
+    only_required_for_messages,
+)
+from pylint.typing import MessageDefinitionTuple
 
-MSGS = {
+if TYPE_CHECKING:
+    from pylint.lint import PyLinter
+
+MSGS: dict[str, MessageDefinitionTuple] = {
     "E1003": (
         "Bad first argument %r given to super()",
         "bad-super-call",
@@ -37,43 +33,41 @@ MSGS = {
 
 
 class NewStyleConflictChecker(BaseChecker):
-    """checks for usage of new style capabilities on old style classes and
-    other new/old styles conflicts problems
+    """Checks for usage of new style capabilities on old style classes and
+    other new/old styles conflicts problems.
+
     * use of property, __slots__, super
     * "super" usage
     """
 
-    __implements__ = (IAstroidChecker,)
-
     # configuration section name
     name = "newstyle"
     # messages
     msgs = MSGS
-    priority = -2
     # configuration options
     options = ()
 
-    @check_messages("bad-super-call")
-    def visit_functiondef(self, node):
-        """check use of super"""
+    @only_required_for_messages("bad-super-call")
+    def visit_functiondef(self, node: nodes.FunctionDef) -> None:
+        """Check use of super."""
         # ignore actual functions or method within a new style class
         if not node.is_method():
             return
-        klass = node.parent.frame()
-        for stmt in node.nodes_of_class(astroid.Call):
+        klass = node.parent.frame(future=True)
+        for stmt in node.nodes_of_class(nodes.Call):
             if node_frame_class(stmt) != node_frame_class(node):
                 # Don't look down in other scopes.
                 continue
 
             expr = stmt.func
-            if not isinstance(expr, astroid.Attribute):
+            if not isinstance(expr, nodes.Attribute):
                 continue
 
             call = expr.expr
             # skip the test if using super
             if not (
-                isinstance(call, astroid.Call)
-                and isinstance(call.func, astroid.Name)
+                isinstance(call, nodes.Call)
+                and isinstance(call.func, nodes.Name)
                 and call.func.name == "super"
             ):
                 continue
@@ -88,8 +82,8 @@ class NewStyleConflictChecker(BaseChecker):
                 # in derived classes
                 arg0 = call.args[0]
                 if (
-                    isinstance(arg0, astroid.Call)
-                    and isinstance(arg0.func, astroid.Name)
+                    isinstance(arg0, nodes.Call)
+                    and isinstance(arg0.func, nodes.Name)
                     and arg0.func.name == "type"
                 ):
                     self.add_message("bad-super-call", node=call, args=("type",))
@@ -99,9 +93,9 @@ class NewStyleConflictChecker(BaseChecker):
                 # in derived classes
                 if (
                     len(call.args) >= 2
-                    and isinstance(call.args[1], astroid.Name)
+                    and isinstance(call.args[1], nodes.Name)
                     and call.args[1].name == "self"
-                    and isinstance(arg0, astroid.Attribute)
+                    and isinstance(arg0, nodes.Attribute)
                     and arg0.attrname == "__class__"
                 ):
                     self.add_message(
@@ -114,7 +108,9 @@ class NewStyleConflictChecker(BaseChecker):
                 except astroid.InferenceError:
                     continue
 
-                if klass is not supcls:
+                # If the supcls is in the ancestors of klass super can be used to skip
+                # a step in the mro() and get a method from a higher parent
+                if klass is not supcls and all(i != supcls for i in klass.ancestors()):
                     name = None
                     # if supcls is not Uninferable, then supcls was inferred
                     # and use its name. Otherwise, try to look
@@ -129,6 +125,5 @@ class NewStyleConflictChecker(BaseChecker):
     visit_asyncfunctiondef = visit_functiondef
 
 
-def register(linter):
-    """required method to auto register this checker """
+def register(linter: PyLinter) -> None:
     linter.register_checker(NewStyleConflictChecker(linter))
