@@ -1,35 +1,39 @@
-# Copyright (c) 2008-2010, 2012-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2014 Brett Cannon <brett@python.org>
-# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
-# Copyright (c) 2015-2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
-# Copyright (c) 2016 Alexander Pervakov <frost.nzcr4@jagmort.com>
-# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
-# Copyright (c) 2019, 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
-# Copyright (c) 2020 Peter Kolbus <peter.kolbus@gmail.com>
-# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2021 Mark Byrne <mbyrnepr2@gmail.com>
-
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
-"""
-  %prog [options] <packages>
+"""Create UML diagrams for classes and modules in <packages>."""
+
+from __future__ import annotations
 
-  create UML diagrams for classes and modules in <packages>
-"""
-import os
-import subprocess
 import sys
+from collections.abc import Sequence
+from typing import NoReturn
 
-from pylint.config import ConfigurationMixIn
+from pylint import constants
+from pylint.config.arguments_manager import _ArgumentsManager
+from pylint.config.arguments_provider import _ArgumentsProvider
+from pylint.lint.utils import fix_import_path
 from pylint.pyreverse import writer
 from pylint.pyreverse.diadefslib import DiadefsHandler
 from pylint.pyreverse.inspector import Linker, project_from_files
-from pylint.pyreverse.utils import insert_default_options
+from pylint.pyreverse.utils import (
+    check_graphviz_availability,
+    check_if_graphviz_supports_format,
+    insert_default_options,
+)
+from pylint.typing import Options
 
-OPTIONS = (
+DIRECTLY_SUPPORTED_FORMATS = (
+    "dot",
+    "vcg",
+    "puml",
+    "plantuml",
+    "mmd",
+    "html",
+)
+
+OPTIONS: Options = (
     (
         "filter-mode",
         dict(
@@ -54,10 +58,11 @@ OPTIONS = (
         "class",
         dict(
             short="c",
-            action="append",
+            action="extend",
             metavar="<class>",
+            type="csv",
             dest="classes",
-            default=[],
+            default=None,
             help="create a class diagram with all classes related to <class>;\
  this uses by default the options -ASmy",
         ),
@@ -69,6 +74,7 @@ OPTIONS = (
             action="store",
             metavar="<ancestor>",
             type="int",
+            default=None,
             help="show <ancestor> generations of ancestor classes not in <projects>",
         ),
     ),
@@ -77,6 +83,7 @@ OPTIONS = (
         dict(
             short="A",
             default=None,
+            action="store_true",
             help="show all ancestors off all classes in <projects>",
         ),
     ),
@@ -87,6 +94,7 @@ OPTIONS = (
             action="store",
             metavar="<association_level>",
             type="int",
+            default=None,
             help="show <association_level> levels of associated classes not in <projects>",
         ),
     ),
@@ -95,6 +103,7 @@ OPTIONS = (
         dict(
             short="S",
             default=None,
+            action="store_true",
             help="show recursively all associated off all associated classes",
         ),
     ),
@@ -113,7 +122,7 @@ OPTIONS = (
             short="m",
             default=None,
             type="yn",
-            metavar="[yn]",
+            metavar="<y or n>",
             help="include module name in representation of classes",
         ),
     ),
@@ -123,8 +132,7 @@ OPTIONS = (
             short="k",
             action="store_true",
             default=False,
-            help="don't show attributes and methods in the class boxes; \
-this disables -f values",
+            help="don't show attributes and methods in the class boxes; this disables -f values",
         ),
     ),
     (
@@ -135,80 +143,97 @@ this disables -f values",
             action="store",
             default="dot",
             metavar="<format>",
-            help="create a *.<format> output file if format available.",
+            type="string",
+            help=(
+                f"create a *.<format> output file if format is available. Available formats are: {', '.join(DIRECTLY_SUPPORTED_FORMATS)}. "
+                f"Any other format will be tried to create by means of the 'dot' command line tool, which requires a graphviz installation."
+            ),
+        ),
+    ),
+    (
+        "colorized",
+        dict(
+            dest="colorized",
+            action="store_true",
+            default=False,
+            help="Use colored output. Classes/modules of the same package get the same color.",
+        ),
+    ),
+    (
+        "max-color-depth",
+        dict(
+            dest="max_color_depth",
+            action="store",
+            default=2,
+            metavar="<depth>",
+            type="int",
+            help="Use separate colors up to package depth of <depth>",
         ),
     ),
     (
         "ignore",
-        {
-            "type": "csv",
-            "metavar": "<file[,file...]>",
-            "dest": "ignore_list",
-            "default": ("CVS",),
-            "help": "Files or directories to be skipped. They "
-            "should be base names, not paths.",
-        },
+        dict(
+            type="csv",
+            metavar="<file[,file...]>",
+            dest="ignore_list",
+            default=constants.DEFAULT_IGNORE_LIST,
+            help="Files or directories to be skipped. They should be base names, not paths.",
+        ),
     ),
     (
         "project",
-        {
-            "default": "",
-            "type": "string",
-            "short": "p",
-            "metavar": "<project name>",
-            "help": "set the project name.",
-        },
+        dict(
+            default="",
+            type="string",
+            short="p",
+            metavar="<project name>",
+            help="set the project name.",
+        ),
     ),
     (
         "output-directory",
-        {
-            "default": "",
-            "type": "string",
-            "short": "d",
-            "action": "store",
-            "metavar": "<output_directory>",
-            "help": "set the output directory path.",
-        },
+        dict(
+            default="",
+            type="path",
+            short="d",
+            action="store",
+            metavar="<output_directory>",
+            help="set the output directory path.",
+        ),
     ),
 )
 
 
-def _check_graphviz_available(output_format):
-    """check if we need graphviz for different output format"""
-    try:
-        subprocess.call(["dot", "-V"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-    except OSError:
-        print(
-            "The output format '%s' is currently not available.\n"
-            "Please install 'Graphviz' to have other output formats "
-            "than 'dot' or 'vcg'." % output_format
-        )
-        sys.exit(32)
+class Run(_ArgumentsManager, _ArgumentsProvider):
+    """Base class providing common behaviour for pyreverse commands."""
 
+    options = OPTIONS
+    name = "pyreverse"
 
-class Run(ConfigurationMixIn):
-    """base class providing common behaviour for pyreverse commands"""
+    # For mypy issue, see https://github.com/python/mypy/issues/10342
+    def __init__(self, args: Sequence[str]) -> NoReturn:  # type: ignore[misc]
+        _ArgumentsManager.__init__(self, prog="pyreverse", description=__doc__)
+        _ArgumentsProvider.__init__(self, self)
 
-    options = OPTIONS  # type: ignore
-
-    def __init__(self, args):
-        ConfigurationMixIn.__init__(self, usage=__doc__)
+        # Parse options
         insert_default_options()
-        args = self.load_command_line_configuration()
-        if self.config.output_format not in ("dot", "vcg"):
-            _check_graphviz_available(self.config.output_format)
+        args = self._parse_command_line_configuration(args)
+
+        if self.config.output_format not in DIRECTLY_SUPPORTED_FORMATS:
+            check_graphviz_availability()
+            print(
+                f"Format {self.config.output_format} is not supported natively. Pyreverse will try to generate it using Graphviz..."
+            )
+            check_if_graphviz_supports_format(self.config.output_format)
 
         sys.exit(self.run(args))
 
-    def run(self, args):
-        """checking arguments and run project"""
+    def run(self, args: list[str]) -> int:
+        """Checking arguments and run project."""
         if not args:
             print(self.help())
             return 1
-        # insert current working directory to the python path to recognize
-        # dependencies to local modules even if cwd is not in the PYTHONPATH
-        sys.path.insert(0, os.getcwd())
-        try:
+        with fix_import_path(args):
             project = project_from_files(
                 args,
                 project_name=self.config.project,
@@ -217,13 +242,7 @@ class Run(ConfigurationMixIn):
             linker = Linker(project, tag=True)
             handler = DiadefsHandler(self.config)
             diadefs = handler.get_diadefs(project, linker)
-        finally:
-            sys.path.pop(0)
-
-        if self.config.output_format == "vcg":
-            writer.VCGWriter(self.config).write(diadefs)
-        else:
-            writer.DotWriter(self.config).write(diadefs)
+        writer.DiagramWriter(self.config).write(diadefs)
         return 0
 
 
